class SessionsController < ApplicationController
  before_action :authenticate_user!, except: [ :index ]
  before_action :authorize_permission, except: %i[index show]

  def index
    @conference = Conference.find_by(slug: params[:conference_slug])
    @sessions = @conference.sessions.includes(:stage).order(:starts_at)

    # Filter by date if provided
    if params[:date].present?
      date = Date.parse(params[:date])
      @sessions = @sessions.where(starts_at: date.beginning_of_day..date.end_of_day)
    end

    # Filter by stage name if provided
    return unless params[:stage].present?
    @sessions = @sessions.joins(:stage).where(stages: { name: params[:stage] })
    # Further filtering options can be added here
  end

  def show
    @conference = Conference.find_by(slug: params[:conference_slug])
    @session = Session.includes(:stage, :filedrop_comments, :filedrop_files).find_by(conference: @conference, ref_id: params[:ref_id])
    @users = User.all
  end

  def update_notes
    conference = Conference.find_by!(slug: params[:conference_slug])
    session = Session.find_by!(conference:, ref_id: params[:ref_id])
    session.notes = params[:note]

    if session.save
      Rails.logger.debug("Updated notes on session #{session.ref_id} to: #{session.notes}")
      Turbo::StreamsChannel.broadcast_replace_to(
        session.conference,
        target: helpers.dom_id(session),
        partial: "sessions/session",
        locals: { session: }
      )
      flash.now[:success] = "Notes saved successfully."
      respond_to do |format|
        format.turbo_stream { render turbo_stream: turbo_stream.replace(helpers.dom_id(session), partial: "sessions/session", locals: { session: }) }
        format.html { redirect_to conference_session_path(session.conference, session), success: "Notes saved successfully." }
      end
    else
      flash.now[:alert] = "Failed to save notes."
      respond_to do |format|
        format.turbo_stream { render turbo_stream: turbo_stream.replace(helpers.dom_id(session), partial: "sessions/session", locals: { session: }), status: :unprocessable_entity }
        format.html { render :show, status: :unprocessable_entity }
      end
    end
  end

  private

  def authorize_permission
    super("manage_assignments")
  end

  def notes_params
    params.require(:session).permit(:notes)
  end
end
